@@ -21,6 +21,7 @@ namespace netxs::events
         forward, // Execute concrete event  first. Forward means from particular to general: 1. event_group::item, 2. event_group::any
         reverse, // Execute global   events first. Reverse means from general to particular: 1. event_group::any , 2. event_group::item
     };
+
     enum class tier
     {
         release, // events: Run forwrad handlers with fixed param. Preserve subscription order.
@@ -30,8 +31,19 @@ namespace netxs::events
         anycast, // events: Run reverse handlers along the entire visual tree. Preserve subscription order.
     };
 
-    template<class V> struct _globals { static std::recursive_mutex              mutex; };
-    template<class V>                          std::recursive_mutex _globals<V>::mutex;
+    struct globals
+    {
+        static auto& mutex()
+        {
+            static auto m = std::recursive_mutex{};
+            return m;
+        }
+        static auto& queue()
+        {
+            static auto q = std::vector<void*>{};
+            return q;
+        }
+    };
 
     struct sync
     {
@@ -40,12 +52,12 @@ namespace netxs::events
         sync             (sync const&) = delete; // deleted copy constructor.
         sync& operator = (sync const&) = delete; // deleted copy assignment operator.
 
-        sync() : lock(_globals<void>::mutex) { }
+        sync() : lock(globals::mutex()) { }
        ~sync() { }
     };
     static auto unique_lock()
     {
-        return std::unique_lock{ _globals<void>::mutex };
+        return std::unique_lock{ globals::mutex() };
     }
     struct try_sync
     {
@@ -56,7 +68,7 @@ namespace netxs::events
         try_sync             (try_sync const&) = delete; // deleted copy constructor.
         try_sync& operator = (try_sync const&) = delete; // deleted copy assignment operator.
 
-        try_sync() : lock(_globals<void>::mutex, std::try_to_lock) { }
+        try_sync() : lock(globals::mutex(), std::try_to_lock) { }
        ~try_sync() { }
     };
 
@@ -106,33 +118,51 @@ namespace netxs::events
     {
         return (event & level_mask(branch, block)) == branch;
     }
-    template<hint event>             constexpr auto offset = level(event) * block;                                  // events: Item/msg bit shift.
-    template<hint event>             constexpr auto parent =          event & ((1 << (offset<event> - block)) - 1); // events: Event group ID.
-    template<hint event>             constexpr auto number =               (event >> (offset<event> - block)) - 1;  // events: Item index inside the group by its ID.
-    template<hint group, auto index> constexpr auto entity = group | ((index + 1) <<  offset<group>);               // events: Event ID of the specified item inside the group.
-
-    template<hint group, auto... index>
-    constexpr auto _instantiate(std::index_sequence<index...>)
+    // events: Return event index inside the group.
+    static constexpr inline auto subindex(hint event)
     {
-        return std::array<hint, sizeof...(index)>{ entity<group, index>... };
+        auto offset = level(event) * block;
+        auto number = (event >> (offset - block)) - 1;
+        return number;
     }
-    template<hint group, auto count> constexpr auto subset = _instantiate<group>(std::make_index_sequence<count>{});
+    // events: Return event id by group + index.
+    static constexpr inline auto makeid(hint group, hint index)
+    {
+        auto offset = level(group) * block;
+        auto entity = group | ((index + 1) <<  offset);
+        return entity;
+    }
+    template<hint Event>             constexpr auto offset = level(Event) * block;                                  // events: Item/msg bit shift.
+    template<hint Event>             constexpr auto parent =          Event & ((1 << (offset<Event> - block)) - 1); // events: Event group ID.
+    template<hint Event>             constexpr auto number =               (Event >> (offset<Event> - block)) - 1;  // events: Item index inside the group by its ID.
+    template<hint Group, auto Index> constexpr auto entity = Group | ((Index + 1) <<  offset<Group>);               // events: Event ID of the specified item inside the group.
+
+    template<hint Group, auto... Index>
+    constexpr auto _instantiate(std::index_sequence<Index...>)
+    {
+        return std::array<hint, sizeof...(Index)>{ entity<Group, Index>... };
+    }
+    template<hint Group, auto Count> constexpr auto subset = _instantiate<Group>(std::make_index_sequence<Count>{});
 
     struct handler
     {
         virtual ~handler() { }
     };
-    using hook = sptr<handler>;
+    struct hook : sptr<handler>
+    {
+        using sptr<handler>::sptr;
+        auto& operator - (si32) { return *this; }
+    };
 
-    template<execution_order ORDER = execution_order::forward>
+    template<execution_order Order = execution_order::forward>
     struct reactor
     {
-        template <class F>
+        template<class F>
         using hndl = std::function<void(F&&)>;
         using list = std::list<wptr<handler>>;
         using vect = std::vector<wptr<handler>>;
 
-        template <class F>
+        template<class F>
         struct wrapper : handler
         {
             hndl<F> proc;
@@ -140,7 +170,8 @@ namespace netxs::events
                 : proc{ proc }
             { }
         };
-        enum branch
+
+        enum class branch
         {
             fullstop,
             nothandled,
@@ -199,7 +230,7 @@ namespace netxs::events
             queue.push_back(event);
             auto head = qcopy.size();
 
-            if constexpr (ORDER == execution_order::forward)
+            if constexpr (Order == execution_order::forward)
             {
                 auto itermask = events::level_mask(event);
                 auto subgroup = event;
@@ -267,13 +298,19 @@ namespace netxs::events
         static wptr<T>                 empty;
         static std::map<id_t, wptr<T>> store;
 
-        // indexer: Return shared_ptr of the object by its id.
+        // indexer: Return sptr of the object by its id.
+        template<class TT = T>
         static auto getref(id_t id)
         {
             auto lock = sync{};
-            return netxs::get_or(store, id, empty).lock();
+            if (auto item_ptr = netxs::get_or(store, id, empty).lock())
+            if (auto real_ptr = std::dynamic_pointer_cast<TT>(item_ptr))
+            {
+                return real_ptr;
+            }
+            return sptr<TT>{};
         }
-        // indexer: Create a new object of the specified subtype and return its shared_ptr.
+        // indexer: Create a new object of the specified subtype and return its sptr.
         template<class TT, class ...Args>
         static auto create(Args&&... args) -> sptr<TT>
         {
@@ -325,6 +362,7 @@ namespace netxs::events
         std::vector<hook> tokens;
 
     public:
+        auto& operator - (si32)    { return *this;                                                           }
         operator bool () const     { return tokens.size();                                                   }
         void  admit(hook&& t)      {        tokens.push_back(std::forward<hook>(t));                         }
         hook& extra()              { return tokens.emplace_back();                                           }
@@ -334,43 +372,69 @@ namespace netxs::events
         void  merge(subs const& m) {        tokens.insert( tokens.end(), m.tokens.begin(), m.tokens.end() ); }
     };
 
-    template<class _parent_type, class _object_type, auto _event_id>
+    template<class Parent_t, class Object_t, auto Event_id>
     struct type_clue
     {
-        using type = _object_type;
-        using base = _parent_type;
-        static constexpr auto id = _event_id;
+        using type = Object_t;
+        using base = Parent_t;
+        static constexpr auto id = Event_id;
         template<class ...Args> constexpr type_clue(Args&&...) { }
         template<class ...Args> static constexpr auto param(Args&&... args) { return type{ std::forward<Args>(args)... }; }
+                                static constexpr auto param(type&&    arg ) { return std::move(arg);                      }
         template<auto N>        static constexpr auto group()               { return events::subset<id, N>;               }
                                 static constexpr auto index()               { return events::number<id>;                  }
     };
 
-    #define SUBMIT(        level, event,        param) bell::template submit<level, decltype( event )>()        = [&] (typename decltype( event )::type &&  param)
-    #define SUBMIT_BYVAL(  level, event,        param) bell::template submit<level, decltype( event )>()        = [=] (typename decltype( event )::type &&  param) mutable
-    #define SUBMIT_T(      level, event, token, param) bell::template submit<level, decltype( event )>( token ) = [&] (typename decltype( event )::type &&  param)
-    #define SUBMIT_T_BYVAL(level, event, token, param) bell::template submit<level, decltype( event )>( token ) = [=] (typename decltype( event )::type &&  param) mutable
-    #define SUBMIT_TV(     level, event, token, proxy) bell::template submit<level, decltype( event )>( token, proxy )
-    #define SUBMIT_V(      level, event,        proxy) bell::template submit<level, decltype( event )>(        proxy )
-    #define SIGNAL(        level, event,        param) bell::template signal<level>(decltype( event )::id, static_cast<typename decltype( event )::type &&>(param))
-    #define SIGNAL_GLOBAL(        event,        param) bell::template signal_global(decltype( event )::id, static_cast<typename decltype( event )::type &&>(param))
-    #define SUBMIT_GLOBAL(        event, token, param) bell::template submit_global<decltype( event )>( token ) = [&] (typename decltype( event )::type &&  param)
+    #define ARGsEVAL_XS(...) __VA_ARGS__
+    #define ARG_EVAL_XS(...) ARGsEVAL_XS(__VA_ARGS__)
+    #define GET_END1_XS(a, b, c, d, e, last, ...) last
+    #define GET_END2_XS(a, b, c, d,    last, ...) last
+
+    #define LISTEN_S(level, event, param              ) bell::template submit<level>( event )           = [&]                  (typename decltype( event )::type&& param)
+    #define LISTEN_T(level, event, param, token       ) bell::template submit<level>( event, token -0 ) = [&]                  (typename decltype( event )::type&& param)
+    #define LISTEN_V(level, event, param, token, byval) bell::template submit<level>( event, token -0 ) = [&, ARG_EVAL_XS byval ] (typename decltype( event )::type&& param) mutable
+    #define LISTEN_X(...) ARG_EVAL_XS(GET_END1_XS(__VA_ARGS__, LISTEN_V, LISTEN_T, LISTEN_S))
+    #if defined(_WIN32)
+        #define LISTEN(...) ARG_EVAL_XS(LISTEN_X(__VA_ARGS__))ARG_EVAL_XS((__VA_ARGS__))
+    #else
+        #define LISTEN(...) LISTEN_X(__VA_ARGS__)(__VA_ARGS__)
+    #endif
+
+    #define SIGNAL_S(level, event, var       ) bell::template signal<level>(decltype( event )::id, static_cast<typename decltype( event )::type &&>(var))
+    #define SIGNAL_N(level, event, var, inits) bell::_saveme(); auto var = event.param ARG_EVAL_XS(inits); bell::_revive()->template signal<level>(decltype( event )::id, static_cast<typename decltype( event )::type &&>(var)); bell::_unlock() // Multi-statement macro. Use with caution.
+    #define SIGNAL_X(...) ARG_EVAL_XS(GET_END2_XS(__VA_ARGS__, SIGNAL_N, SIGNAL_S))
+    #if defined(_WIN32)
+        #define SIGNAL(...) ARG_EVAL_XS(SIGNAL_X(__VA_ARGS__))ARG_EVAL_XS((__VA_ARGS__))
+    #else
+        #define SIGNAL(...) SIGNAL_X(__VA_ARGS__)(__VA_ARGS__)
+    #endif
+
+    #define RISEUP_S(level, event, var       ) base::template riseup<level>(event, var)
+    #define RISEUP_N(level, event, var, inits) base::_saveme(); auto var = event.param ARG_EVAL_XS(inits); static_cast<base*>(bell::_revive())->template riseup<level>(event, var); bell::_unlock() // Multi-statement macro. Use with caution.
+    #define RISEUP_X(...) ARG_EVAL_XS(GET_END2_XS(__VA_ARGS__, RISEUP_N, RISEUP_S))
+    #if defined(_WIN32)
+        #define RISEUP(...) ARG_EVAL_XS(RISEUP_X(__VA_ARGS__))ARG_EVAL_XS((__VA_ARGS__))
+    #else
+        #define RISEUP(...) RISEUP_X(__VA_ARGS__)(__VA_ARGS__)
+    #endif
 
     //todo deprecated?
-    #define SUBMIT_AND_RUN_T(level, event, token, param, arg) bell::template submit2<level,decltype( event )>( arg, token ) = [&] (typename decltype( event )::type && param)
-    #define SUBMIT_AND_RUN(  level, event,        param, arg) bell::template submit2<level,decltype( event )>( arg        ) = [&] (typename decltype( event )::type && param)
+    //#define LISTEN_AND_RUN_T(level, event, token, param, arg) bell::template submit2<level,decltype( event )>( arg, token ) = [&] (typename decltype( event )::type && param)
+    //#define LISTEN_AND_RUN(  level, event,        param, arg) bell::template submit2<level,decltype( event )>( arg        ) = [&] (typename decltype( event )::type && param)
+    //#define SIGNAL_GLOBAL(        event, param              ) bell::template signal_global(decltype( event )::id, static_cast<typename decltype( event )::type &&>(param))
+    //#define LISTEN_GLOBAL(        event, param, token       ) bell::template submit_global( event, token -0 ) = [&]                  (typename decltype( event )::type&& param)
 
     #define EVENTPACK( name, base ) using _group_type = name; \
                                     static constexpr auto _counter_base = __COUNTER__; \
-                                    public: static constexpr auto any = type_clue<_group_type, decltype(base)::type, decltype(base)::id>
-    #define  EVENT_XS( name, type ) }; static constexpr auto name = type_clue<_group_type, type, decltype(any)::id | ((__COUNTER__ - _counter_base) << netxs::events::offset<decltype(any)::id>)>{ 777
+                                    public: static constexpr auto any = netxs::events::type_clue<_group_type, decltype(base)::type, decltype(base)::id>
+    #define  EVENT_XS( name, type ) }; static constexpr auto name = netxs::events::type_clue<_group_type, type, decltype(any)::id | ((__COUNTER__ - _counter_base) << netxs::events::offset<decltype(any)::id>)>{ 777
     #define  GROUP_XS( name, type ) EVENT_XS( _##name, type )
     #define SUBSET_XS( name )       }; class name { EVENTPACK( name, _##name )
     #define  INDEX_XS(  ... )       }; template<auto N> static constexpr \
                                     auto _ = std::get<N>( std::tuple{ __VA_ARGS__ } ); \
                                     private: static constexpr auto _dummy = { 777
 
-    class bell;
+    struct bell;
     using ftor = std::function<bool(sptr<bell>)>;
 
     struct ref_count_t
@@ -385,7 +449,7 @@ namespace netxs::events
     {
         struct root
         {
-            static constexpr auto root_event = type_clue<root, void, 0>{};
+            static constexpr auto root_event = type_clue<root, si32, 0>{};
             EVENTPACK( root, root_event )
             {
                 EVENT_XS( dtor   , const id_t ),
@@ -399,9 +463,8 @@ namespace netxs::events
     }
 
     // events: Event x-mitter.
-    class bell : public indexer<bell>
+    struct bell : public indexer<bell>
     {
-    public:
         static constexpr auto noid = std::numeric_limits<id_t>::max();
         subs tracker;
 
@@ -409,19 +472,23 @@ namespace netxs::events
         using fwd_reactor = reactor<execution_order::forward>;
         using rev_reactor = reactor<execution_order::reverse>;
 
-        template<class V> struct _globals { static fwd_reactor general; };
+        static fwd_reactor& _general()
+        {
+            static auto g = fwd_reactor{};
+            return g;
+        }
 
-        fwd_reactor& general{ _globals<void>::general };
+        fwd_reactor& general{ _general() };
         fwd_reactor  release;
         fwd_reactor  request;
         rev_reactor  preview;
         rev_reactor  anycast;
 
         //todo deprecated?
-        template<tier TIER, class EVENT>
+        template<tier Tier, class Event>
         struct submit_helper2
         {
-            using type = typename EVENT::type;
+            using type = typename Event::type;
             bell& owner;
             type& p;
             submit_helper2(bell& owner, type& p)
@@ -431,15 +498,15 @@ namespace netxs::events
             template<class F>
             void operator = (F h)
             {
-                owner.submit<TIER, EVENT>(h);
+                owner.submit<Tier>(Event{}, h);
                 h(static_cast<type&&>(p));
             }
         };
         //todo deprecated?
-        template<tier TIER, class EVENT>
+        template<tier Tier, class Event>
         struct submit_helper2_token
         {
-            using type = typename EVENT::type;
+            using type = typename Event::type;
             bell& owner;
             type& p;
             hook& token;
@@ -451,11 +518,11 @@ namespace netxs::events
             template<class F>
             void operator = (F h)
             {
-                owner.submit<TIER, EVENT>(token, h);
+                owner.submit<Tier>(Event{}, token, h);
                 h(static_cast<type&&>(p));
             }
         };
-        template<tier TIER, class EVENT>
+        template<tier Tier, class Event>
         struct submit_helper
         {
             bell& owner;
@@ -465,10 +532,10 @@ namespace netxs::events
             template<class F>
             void operator = (F h)
             {
-                owner.submit<TIER, EVENT>(h);
+                owner.submit<Tier>(Event{}, h);
             }
         };
-        template<tier TIER, class EVENT>
+        template<tier Tier, class Event>
         struct submit_helper_token
         {
             bell& owner;
@@ -480,10 +547,10 @@ namespace netxs::events
             template<class F>
             void operator = (F h)
             {
-                owner.submit<TIER, EVENT>(token, h);
+                owner.submit<Tier>(Event{}, token, h);
             }
         };
-        template<class EVENT>
+        template<class Event>
         struct submit_helper_token_global
         {
             hook& token;
@@ -493,90 +560,122 @@ namespace netxs::events
             template<class F>
             void operator = (F h)
             {
-                token = _globals<void>::general.subscribe(EVENT::id, std::function<void(typename EVENT::type &&)>{ h });
+                token = _general().subscribe(Event::id, std::function<void(typename Event::type &&)>{ h });
             }
         };
 
     public:
         //todo deprecated?
-        template<tier TIER, class EVENT> auto submit2(typename EVENT::type & p)               { return submit_helper2      <TIER, EVENT>(*this, p);                 }
-        template<tier TIER, class EVENT> auto submit2(typename EVENT::type & p, subs& tokens) { return submit_helper2_token<TIER, EVENT>(*this, p, tokens.extra()); }
+        template<tier Tier, class Event> auto submit2(typename Event::type & p)               { return submit_helper2      <Tier, Event>(*this, p);                 }
+        template<tier Tier, class Event> auto submit2(typename Event::type & p, subs& tokens) { return submit_helper2_token<Tier, Event>(*this, p, tokens.extra()); }
 
-        template<tier TIER, class EVENT> auto submit()             { return submit_helper      <TIER, EVENT>(*this);                 }
-        template<tier TIER, class EVENT> auto submit(hook& token)  { return submit_helper_token<TIER, EVENT>(*this, token);          }
-        template<tier TIER, class EVENT> auto submit(subs& tokens) { return submit_helper_token<TIER, EVENT>(*this, tokens.extra()); }
-        template<tier TIER, class EVENT>
-        void submit(std::function<void(typename EVENT::type &&)> handler)
+        template<tier Tier, class Event> auto submit(Event)               { return submit_helper      <Tier, Event>(*this);                 }
+        template<tier Tier, class Event> auto submit(Event, si32)         { return submit_helper      <Tier, Event>(*this);                 }
+        template<tier Tier, class Event> auto submit(Event, hook& token)  { return submit_helper_token<Tier, Event>(*this, token);          }
+        template<tier Tier, class Event> auto submit(Event, subs& tokens) { return submit_helper_token<Tier, Event>(*this, tokens.extra()); }
+        template<tier Tier, class Event>
+        void submit(Event, std::function<void(typename Event::type &&)> handler)
         {
-                 if constexpr (TIER == tier::preview) tracker.admit(preview.subscribe(EVENT::id, handler));
-            else if constexpr (TIER == tier::general) tracker.admit(general.subscribe(EVENT::id, handler));
-            else if constexpr (TIER == tier::request) tracker.admit(request.subscribe(EVENT::id, handler));
-            else if constexpr (TIER == tier::release) tracker.admit(release.subscribe(EVENT::id, handler));
-            else                                      tracker.admit(anycast.subscribe(EVENT::id, handler));
+                 if constexpr (Tier == tier::preview) tracker.admit(preview.subscribe(Event::id, handler));
+            else if constexpr (Tier == tier::general) tracker.admit(general.subscribe(Event::id, handler));
+            else if constexpr (Tier == tier::request) tracker.admit(request.subscribe(Event::id, handler));
+            else if constexpr (Tier == tier::release) tracker.admit(release.subscribe(Event::id, handler));
+            else                                      tracker.admit(anycast.subscribe(Event::id, handler));
         }
-        template<tier TIER, class EVENT>
-        void submit(hook& token, std::function<void(typename EVENT::type &&)> handler)
+        template<tier Tier, class Event>
+        void submit(Event, hook& token, std::function<void(typename Event::type &&)> handler)
         {
-                 if constexpr (TIER == tier::preview) token = preview.subscribe(EVENT::id, handler);
-            else if constexpr (TIER == tier::general) token = general.subscribe(EVENT::id, handler);
-            else if constexpr (TIER == tier::request) token = request.subscribe(EVENT::id, handler);
-            else if constexpr (TIER == tier::release) token = release.subscribe(EVENT::id, handler);
-            else                                      token = anycast.subscribe(EVENT::id, handler);
+                 if constexpr (Tier == tier::preview) token = preview.subscribe(Event::id, handler);
+            else if constexpr (Tier == tier::general) token = general.subscribe(Event::id, handler);
+            else if constexpr (Tier == tier::request) token = request.subscribe(Event::id, handler);
+            else if constexpr (Tier == tier::release) token = release.subscribe(Event::id, handler);
+            else                                      token = anycast.subscribe(Event::id, handler);
         }
-        template<tier TIER, class F>
+        template<tier Tier, class F>
         auto signal(hint event, F&& data)
         {
-                 if constexpr (TIER == tier::preview) return preview.notify(event, std::forward<F>(data));
-            else if constexpr (TIER == tier::general) return general.notify(event, std::forward<F>(data));
-            else if constexpr (TIER == tier::request) return request.notify(event, std::forward<F>(data));
-            else if constexpr (TIER == tier::release) return release.notify(event, std::forward<F>(data));
-            else            /* TIER == tier::anycast */
+                 if constexpr (Tier == tier::preview) return preview.notify(event, std::forward<F>(data));
+            else if constexpr (Tier == tier::general) return general.notify(event, std::forward<F>(data));
+            else if constexpr (Tier == tier::request) return request.notify(event, std::forward<F>(data));
+            else if constexpr (Tier == tier::release) return release.notify(event, std::forward<F>(data));
+            else            /* Tier == tier::anycast */
             {
                 auto root = gettop();
-                ftor proc = [&](auto boss_ptr) -> bool
+                auto proc = ftor{ [&](auto boss_ptr)
                 {
                     boss_ptr->anycast.notify(event, std::forward<F>(data));
                     return true;
-                };
+                }};
                 return root->release.notify(userland::root::cascade.id, proc);
             }
         }
-        template<class EVENT> static auto submit_global(hook& token)           { return submit_helper_token_global<EVENT>(token); }
-        template<class EVENT> static auto submit_global(subs& tokens)          { return submit_helper_token_global<EVENT>(tokens.extra()); }
-        template<class F>     static auto signal_global(hint  event, F&& data) { return _globals<void>::general.notify(event, std::forward<F>(data)); }
+        //todo deprecated
+        //template<class F>     static auto signal_global(hint event, F&& data) { return _globals<void>::general.notify(event, std::forward<F>(data)); }
+        //template<class Event> static auto submit_global(Event, hook& token)   { return submit_helper_token_global<Event>(token); }
+        //template<class Event> static auto submit_global(Event, subs& tokens)  { return submit_helper_token_global<Event>(tokens.extra()); }
         // bell: Return initial event of the current event execution branch.
-        template<tier TIER>
+        template<tier Tier>
         auto protos()
         {
-                 if constexpr (TIER == tier::preview) return preview.queue.empty() ? hint{} : preview.queue.back();
-            else if constexpr (TIER == tier::general) return general.queue.empty() ? hint{} : general.queue.back();
-            else if constexpr (TIER == tier::request) return request.queue.empty() ? hint{} : request.queue.back();
-            else if constexpr (TIER == tier::release) return release.queue.empty() ? hint{} : release.queue.back();
+                 if constexpr (Tier == tier::preview) return preview.queue.empty() ? hint{} : preview.queue.back();
+            else if constexpr (Tier == tier::general) return general.queue.empty() ? hint{} : general.queue.back();
+            else if constexpr (Tier == tier::request) return request.queue.empty() ? hint{} : request.queue.back();
+            else if constexpr (Tier == tier::release) return release.queue.empty() ? hint{} : release.queue.back();
             else                                      return anycast.queue.empty() ? hint{} : anycast.queue.back();
         }
-        template<tier TIER, class EVENT> auto protos(EVENT) { return bell::protos<TIER>() == EVENT::id; }
-        template<tier TIER>
+        template<tier Tier, class Event> auto protos(Event) { return bell::protos<Tier>() == Event::id; }
+        template<tier Tier>
         auto& router()
         {
-                 if constexpr (TIER == tier::preview) return preview;
-            else if constexpr (TIER == tier::general) return general;
-            else if constexpr (TIER == tier::request) return request;
-            else if constexpr (TIER == tier::release) return release;
+                 if constexpr (Tier == tier::preview) return preview;
+            else if constexpr (Tier == tier::general) return general;
+            else if constexpr (Tier == tier::request) return request;
+            else if constexpr (Tier == tier::release) return release;
             else                                      return anycast;
         }
-        template<tier TIER>
+        template<tier Tier>
         void expire()
         {
-                 if constexpr (TIER == tier::preview) return preview.stop();
-            else if constexpr (TIER == tier::general) return general.stop();
-            else if constexpr (TIER == tier::request) return request.stop();
-            else if constexpr (TIER == tier::release) return release.stop();
+                 if constexpr (Tier == tier::preview) return preview.stop();
+            else if constexpr (Tier == tier::general) return general.stop();
+            else if constexpr (Tier == tier::request) return request.stop();
+            else if constexpr (Tier == tier::release) return release.stop();
             else                                      return anycast.stop();
         }
+        // bell: Sync with UI thread.
+        template<class P>
+        void trysync(flag& active, P proc)
+        {
+            while (active)
+            {
+                if (auto guard = netxs::events::try_sync{})
+                {
+                    proc();
+                    break;
+                }
+                std::this_thread::yield();
+            }            
+        }
+        void _saveme()
+        {
+            globals::mutex().lock();
+            globals::queue().push_back(this);
+        }
+        static auto _revive()
+        {
+            auto& queue = globals::queue();
+            auto ptr = queue.back();
+                       queue.pop_back();
+            return static_cast<bell*>(ptr);
+        }
+        static void _unlock()
+        {
+            globals::mutex().unlock();
+        }
 
         bell()
         {
-            SUBMIT_GLOBAL(userland::root::cleanup, tracker.extra(), counter)
+            LISTEN(tier::general, userland::root::cleanup, counter)
             {
                 counter.obj_count++;
                 preview.cleanup(counter.ref_count, counter.del_count);
@@ -585,9 +684,10 @@ namespace netxs::events
                 anycast.cleanup(counter.ref_count, counter.del_count);
             };
         }
-       ~bell() { SIGNAL(tier::release, userland::root::dtor, id); }
-
-        virtual void global(twod& coor) { } // bell: Recursively calculate global coordinate.
+       ~bell()
+        {
+            SIGNAL(tier::release, userland::root::dtor, id);
+        }
         virtual sptr<bell> gettop() { return sptr<bell>(this, noop{}); } // bell: Recursively find the root of the visual tree.
     };
 
@@ -613,6 +713,16 @@ namespace netxs::events
             }
         });
     }
-
-    template<class T> bell::fwd_reactor bell::_globals<T>::general;
+    void dequeue()
+    {
+        auto& agent = _agent<void>();
+        agent.stop();
+    }
+}
+namespace netxs
+{
+    using netxs::events::bell;
+    using netxs::events::subs;
+    using netxs::events::tier;
+    using netxs::events::hook;
 }
\ No newline at end of file
