@@ -4,7 +4,6 @@
 #pragma once
 
 #include "directvt.hpp"
-#include "logger.hpp"
 
 namespace netxs::ui
 {
@@ -13,12 +12,13 @@ namespace netxs::ui
 
     class poly
     {
+        cell basis;
         cell grade[256];
 
     public:
         poly() = default;
-
         poly(cell const& basis)
+            : basis{ basis }
         {
             recalc(basis);
         }
@@ -35,7 +35,8 @@ namespace netxs::ui
             }
         }
 
-        cell const& operator [] (uint8_t k) const
+        operator auto& () const { return basis; }
+        auto& operator [] (byte k) const
         {
             return grade[k];
         }
@@ -80,9 +81,9 @@ namespace netxs::ui
         static void exec_zz(flow& f, si32 a) { f.zz( ); }
 
         // flow: Draw commands (customizable).
-        template<ansi::fn CMD>
-        static void exec_dc(flow& f, si32 a) { if (f.custom) f.custom(CMD, a); }
-        //static void exec_dc(flow& f, si32 a) { f.custom(CMD, a); }
+        template<ansi::fn Cmd>
+        static void exec_dc(flow& f, si32 a) { if (f.custom) f.custom(Cmd, a); }
+        //static void exec_dc(flow& f, si32 a) { f.custom(Cmd, a); }
 
         // flow: Abstract handler
         //       ansi::fn::ed
@@ -111,19 +112,19 @@ namespace netxs::ui
         };
 
         // flow: Main cursor forwarding proc.
-        template<bool SPLIT, bool WRAP, bool RtoL, bool ReLF, class T, class P>
+        template<bool Split, bool Wrap, bool RtoL, bool ReLF, class T, class P>
         void output(T const& block, P print)
         {
             textline.coor = caretpos;
 
             auto printout = rect{};
             auto outwidth = si32{};
-            if constexpr (WRAP)
+            if constexpr (Wrap)
             {
                 printout = textline.trunc(viewrect.size);
                 outwidth = printout.coor.x + printout.size.x - textline.coor.x;
 
-                if constexpr (!SPLIT)
+                if constexpr (!Split)
                 if (printout.size.x > 1)
                 {
                     auto p = curpoint + printout.size.x - 1;
@@ -158,7 +159,7 @@ namespace netxs::ui
             if constexpr (!std::is_same_v<P, noop>)
             {
                 //todo margins don't apply to unwrapped text
-                //auto imprint = WRAP ? printout
+                //auto imprint = Wrap ? printout
                 //                    : viewrect.clip(printout);
                 if (printout)
                 {
@@ -175,7 +176,7 @@ namespace netxs::ui
         auto middle() { return (viewrect.size.x >> 1) - (textline.size.x >> 1); }
         void autocr() { if (caretpos.x >= caret_mx) flow::nl(highness); }
 
-        template<bool SPLIT, bool RtoL, bool ReLF, class T, class P>
+        template<bool Split, bool RtoL, bool ReLF, class T, class P>
         void centred(T const& block, P print)
         {
             while (textline.size.x > 0)
@@ -184,33 +185,33 @@ namespace netxs::ui
                 auto axis = textline.size.x >= caret_mx ? 0
                                                         : middle();
                 flow::ax(axis);
-                output<SPLIT, true, RtoL, ReLF>(block, print);
+                output<Split, true, RtoL, ReLF>(block, print);
             }
         }
-        template<bool SPLIT, bool RtoL, bool ReLF, class T, class P>
+        template<bool Split, bool RtoL, bool ReLF, class T, class P>
         void wrapped(T const& block, P print)
         {
             while (textline.size.x > 0)
             {
                 autocr();
-                output<SPLIT, true, RtoL, ReLF>(block, print);
+                output<Split, true, RtoL, ReLF>(block, print);
             }
         }
-        template<bool SPLIT, bool RtoL, bool ReLF, class T, class P>
+        template<bool Split, bool RtoL, bool ReLF, class T, class P>
         void trimmed(T const& block, P print)
         {
             if (textline.size.x > 0)
             {
                 if (centered) flow::ax(middle());
-                output<SPLIT, faux, RtoL, ReLF>(block, print);
+                output<Split, faux, RtoL, ReLF>(block, print);
             }
         }
-        template<bool SPLIT, bool RtoL, bool ReLF, class T, class P>
+        template<bool Split, bool RtoL, bool ReLF, class T, class P>
         void proceed(T const& block, P print)
         {
-            if (iswrapln) if (centered) centred<SPLIT, RtoL, ReLF>(block, print);
-                          else          wrapped<SPLIT, RtoL, ReLF>(block, print);
-            else                        trimmed<SPLIT, RtoL, ReLF>(block, print);
+            if (iswrapln) if (centered) centred<Split, RtoL, ReLF>(block, print);
+                          else          wrapped<Split, RtoL, ReLF>(block, print);
+            else                        trimmed<Split, RtoL, ReLF>(block, print);
         }
 
         std::function<void(ansi::fn cmd, si32 arg)> custom; // flow: Draw commands (customizable).
@@ -235,13 +236,17 @@ namespace netxs::ui
         void  minmax(twod const& p)  { boundary |= p;        } // flow: Register twod.
         void  minmax(rect const& r)  { boundary |= r;        } // flow: Register rect.
 
-        // flow: Split specified textblock on the substrings
-        //       and place it to the form by the specified proc.
-        template<bool SPLIT, class T, class P = noop>
-        void compose(T const& block, P print = P())
+        // flow: Sync paragraph style.
+        template<class T>
+        auto sync(T const& block)
         {
             combine(runstyle, block.style);
-
+        }
+        // flow: Split specified textblock on the substrings
+        //       and place it to the form by the specified proc.
+        template<bool Split, class T, class P = noop>
+        void compose(T const& block, P print = P())
+        {
             auto block_size = block.size();
             textsize = getlen(block_size);
             if (textsize)
@@ -262,13 +267,13 @@ namespace netxs::ui
 
                 if (arighted)
                 {
-                    if (isrlfeed) proceed<SPLIT, true, true>(block, print);
-                    else          proceed<SPLIT, true, faux>(block, print);
+                    if (isrlfeed) proceed<Split, true, true>(block, print);
+                    else          proceed<Split, true, faux>(block, print);
                 }
                 else
                 {
-                    if (isrlfeed) proceed<SPLIT, faux, true>(block, print);
-                    else          proceed<SPLIT, faux, faux>(block, print);
+                    if (isrlfeed) proceed<Split, faux, true>(block, print);
+                    else          proceed<Split, faux, faux>(block, print);
                 }
             }
         }
@@ -283,37 +288,39 @@ namespace netxs::ui
             }
             return flow::up();
         }
-        template<bool SPLIT = true, class T>
+        template<bool Split = true, class T>
         void go(T const& block)
         {
-            compose<SPLIT>(block);
+            compose<Split>(block);
         }
-        template<bool SPLIT = true, class T, class P = noop>
+        template<bool Split = true, class T, class P = noop>
         void go(T const& block, core& canvas, P printfx = P())
         {
-            compose<SPLIT>(block, [&](auto const& coord, auto const& subblock, auto isr_to_l)
+            compose<Split>(block, [&](auto const& coord, auto const& subblock, auto isr_to_l)
                                   {
                                       canvas.text(coord, subblock, isr_to_l, printfx);
                                   });
         }
-        template<bool USE_LOCUS = true, class T, class P = noop>
+        template<bool UseLocus = true, class T, class P = noop>
         auto print(T const& block, core& canvas, P printfx = P())
         {
+            sync(block);
             auto coor = std::invoke_result_t<decltype(&flow::cp), flow>{};
 
-            if constexpr (USE_LOCUS) coor = forward(block);
-            else                     coor = flow::cp();
+            if constexpr (UseLocus) coor = forward(block);
+            else                    coor = flow::cp();
 
             go<faux>(block, canvas, printfx);
             return coor;
         }
-        template<bool USE_LOCUS = true, class T>
+        template<bool UseLocus = true, class T>
         auto print(T const& block)
         {
+            sync(block);
             auto coor = std::invoke_result_t<decltype(&flow::cp), flow>{};
 
-            if constexpr (USE_LOCUS) coor = forward(block);
-            else                     coor = flow::cp();
+            if constexpr (UseLocus) coor = forward(block);
+            else                    coor = flow::cp();
 
             go<faux>(block);
             return coor;
@@ -433,7 +440,7 @@ namespace netxs::ui
         { }
 
         constexpr
-        auto substr(si32 begin, si32 piece = netxs::maxsi32) const
+        auto substr(si32 begin, si32 piece = si32max) const
         {
             auto w = basis.size().x;
             auto a = start + std::max(begin, 0);
@@ -507,7 +514,7 @@ namespace netxs::ui
 
         auto length() const                                     { return size().x;                         }
         auto shadow() const                                     { return shot{ *this };                    }
-        auto substr(si32 at, si32 width = netxs::maxsi32) const { return shadow().substr(at, width);       }
+        auto substr(si32 at, si32 width = netxs::si32max) const { return shadow().substr(at, width);       }
         void trimto(si32 max_size)                              { if (length() > max_size) crop(max_size); }
         void resize(si32 oversize)                              { if (oversize > length()) crop(oversize); }
         auto empty()
@@ -530,12 +537,12 @@ namespace netxs::ui
             if (max_size && max_size < new_size) new_size = max_size;
             if (new_size != length()) crop(new_size);
         }
-        template<bool AUTOGROW = faux>
+        template<bool AutoGrow = faux>
         void splice(si32 at, si32 count, cell const& blank)
         {
             if (count <= 0) return;
             auto len = length();
-            if constexpr (AUTOGROW)
+            if constexpr (AutoGrow)
             {
                 resize(at + count);
             }
@@ -560,8 +567,8 @@ namespace netxs::ui
             auto src = fragment.data();
             while (dst != end) fuse(*dst++, *src++);
         }
-        template<bool Copy = faux, class SRC_IT, class DST_IT, class Shader>
-        static void forward_fill_proc(SRC_IT data, DST_IT dest, DST_IT tail, Shader fuse)
+        template<bool Copy = faux, class SrcIt, class DstIt, class Shader>
+        static void forward_fill_proc(SrcIt data, DstIt dest, DstIt tail, Shader fuse)
         {
             if constexpr (Copy)
             {
@@ -575,7 +582,7 @@ namespace netxs::ui
             //  + while (--wide)
             //    {
             //        /* IT IS UNSAFE IF REALLOCATION OCCURS. BOOK ALWAYS */
-            //        lyric.emplace_back(cluster, ui::whitespace);
+            //        lyric.emplace_back(cluster, whitespace);
             //    }
             //  + convert front into the screen-like sequence (unfold, remmove zerospace chars)
 
@@ -602,7 +609,7 @@ namespace netxs::ui
                 else if (w > 2)
                 {
                     // Forbid using super wide characters until terminal emulators support the fragmentation attribute.
-                    c.txt(utf::REPLACEMENT_CHARACTER_UTF8_VIEW);
+                    c.txt(utf::replacement);
                     do fuse(*dest++, c);
                     while (--w && dest != tail + 1);
                 }
@@ -613,11 +620,11 @@ namespace netxs::ui
                 auto w = c.wdt();
                      if (w == 1) fuse(*dest, c);
                 else if (w == 2) fuse(*dest, c.wdt(3));
-                else if (w >  2) fuse(*dest, c.txt(utf::REPLACEMENT_CHARACTER_UTF8_VIEW));
+                else if (w >  2) fuse(*dest, c.txt(utf::replacement));
             }
         }
-        template<bool Copy = faux, class SRC_IT, class DST_IT, class Shader>
-        static void unlimit_fill_proc(SRC_IT data, si32 size, DST_IT dest, DST_IT tail, si32 back, Shader fuse)
+        template<bool Copy = faux, class SrcIt, class DstIt, class Shader>
+        static void unlimit_fill_proc(SrcIt data, si32 size, DstIt dest, DstIt tail, si32 back, Shader fuse)
         {
             if constexpr (Copy)
             {
@@ -631,7 +638,7 @@ namespace netxs::ui
             //  + while (--wide)
             //    {
             //        /* IT IS UNSAFE IF REALLOCATION OCCURS. BOOK ALWAYS */
-            //        lyric.emplace_back(cluster, ui::whitespace);
+            //        lyric.emplace_back(cluster, whitespace);
             //    }
             //  + convert front into the screen-like sequence (unfold, remmove zerospace chars)
 
@@ -663,14 +670,14 @@ namespace netxs::ui
                 else if (w > 2)
                 {
                     // Forbid using super wide characters until terminal emulators support the fragmentation attribute.
-                    //c.txt(utf::REPLACEMENT_CHARACTER_UTF8_VIEW);
+                    //c.txt(utf::replacement);
                     //do set(c);
                     //while (--w && size > 0);
                 }
             }
         }
-        template<bool Copy = faux, class SRC_IT, class DST_IT, class Shader>
-        static void reverse_fill_proc(SRC_IT data, DST_IT dest, DST_IT tail, Shader fuse)
+        template<bool Copy = faux, class SrcIt, class DstIt, class Shader>
+        static void reverse_fill_proc(SrcIt data, DstIt dest, DstIt tail, Shader fuse)
         {
             if constexpr (Copy)
             {
@@ -684,7 +691,7 @@ namespace netxs::ui
             //  + while (--wide)
             //    {
             //        /* IT IS UNSAFE IF REALLOCATION OCCURS. BOOK ALWAYS */
-            //        lyric.emplace_back(cluster, ui::whitespace);
+            //        lyric.emplace_back(cluster, whitespace);
             //    }
             //  + convert front into the screen-like sequence (unfold, remmove zerospace chars)
 
@@ -711,7 +718,7 @@ namespace netxs::ui
                 else if (w > 2)
                 {
                     // Forbid using super wide characters until terminal emulators support the fragmentation attribute.
-                    //c.txt(utf::REPLACEMENT_CHARACTER_UTF8_VIEW);
+                    //c.txt(utf::replacement);
                     //do *--dest = c;
                     //while (--w && dest != tail - 1);
                 }
@@ -722,7 +729,7 @@ namespace netxs::ui
                 auto w = c.wdt();
                      if (w == 1) fuse(*--dest, c);
                 else if (w == 2) fuse(*--dest, c.wdt(3));
-                else if (w >  2) fuse(*--dest, c.txt(utf::REPLACEMENT_CHARACTER_UTF8_VIEW));
+                else if (w >  2) fuse(*--dest, c.txt(utf::replacement));
             }
         }
         // rich: Splice proto with auto grow.
@@ -1008,7 +1015,7 @@ namespace netxs::ui
         si32 caret = 0; // para: Cursor position inside lyric.
         ui32 index = 0;
         writ locus;
-        corx lyric = std::make_shared<rich>();
+        corx lyric = ptr::shared<rich>();
 
         using parser::parser;
         para()                         = default;
@@ -1027,7 +1034,7 @@ namespace netxs::ui
 
         operator writ const& () const { return locus; }
 
-        void decouple() { lyric = std::make_shared<rich>(*lyric); } // para: Make canvas isolated copy.
+        void decouple() { lyric = ptr::shared<rich>(*lyric); } // para: Make canvas isolated copy.
         void  content(rich& r){ *lyric = r; caret = r.length(); } // para: Set paragraph content.
         auto& content() const { return *lyric; } // para: Return paragraph content.
         shot   shadow() const { return *lyric; } // para: Return paragraph shadow.
@@ -1062,6 +1069,20 @@ namespace netxs::ui
             lyric->splice(caret, count, proto, cell::shaders::full);
             caret += count;
         }
+        //todo unify: see ui::page::post
+        void post(utf::frag const& cluster)
+        {
+            if (cluster.attr.cdpoint == 0) // Override null character - set a narrow width.
+            {
+                auto c = cluster;
+                c.attr.ucwidth = netxs::unidata::widths::slim;
+                ansi::parser::post(c);
+            }
+            else
+            {
+                ansi::parser::post(cluster);
+            }
+        }
         void id(ui32 newid) { index = newid; }
         auto id() const     { return index;  }
 
@@ -1310,8 +1331,8 @@ namespace netxs::ui
                         insert(*iter2);
                         return true;
                     }
-                    if ((iter1++)->wdt() == 2 && iter1 != end_1 && (iter1++)->wdt() != 3) log("para: corrupted glyph");
-                    if ((iter2++)->wdt() == 2 && iter2 != end_2 && (iter2++)->wdt() != 3) log("para: corrupted glyph");
+                    if ((iter1++)->wdt() == 2 && iter1 != end_1 && (iter1++)->wdt() != 3) log(prompt::para, "Corrupted glyph");
+                    if ((iter2++)->wdt() == 2 && iter2 != end_2 && (iter2++)->wdt() != 3) log(prompt::para, "Corrupted glyph");
                 }
             }
             return faux;
@@ -1468,12 +1489,14 @@ namespace netxs::ui
         using list = std::list<sptr<para>>;
         using iter = list::iterator;
         using pmap = std::map<si32, wptr<para>>;
+        using redo = std::list<std::pair<deco, ansi::mark>>;
 
     public:
         ui32 index = {};              // page: Current paragraph id.
-        list batch = { std::make_shared<para>(index) }; // page: Paragraph list.
+        list batch = { ptr::shared<para>(index) }; // page: Paragraph list.
         iter layer = batch.begin();   // page: Current paragraph.
-        pmap parts;                   // page: Paragraph index.
+        pmap parts = {};              // page: Paragraph index.
+        redo stack = {};              // paпу: Style state stack.
 
         //todo use ring
         si32 limit = std::numeric_limits<si32>::max(); // page: Paragraphs number limit.
@@ -1507,14 +1530,20 @@ namespace netxs::ui
         static void parser_config(T& vt)
         {
             using namespace netxs::ansi;
-            vt.intro[ctrl::CR ]              = VT_PROC{ q.pop_if(ctrl::EOL); p->task({ fn::nl,1 }); };
-            vt.intro[ctrl::TAB]              = VT_PROC{ p->task({ fn::tb, q.pop_all(ctrl::TAB) }); };
-            vt.intro[ctrl::EOL]              = VT_PROC{ p->task({ fn::nl, q.pop_all(ctrl::EOL) }); };
-            vt.csier.table[CSI__ED]          = VT_PROC{ p->task({ fn::ed, q(0) }); }; // CSI Ps J
-            vt.csier.table[CSI__EL]          = VT_PROC{ p->task({ fn::el, q(0) }); }; // CSI Ps K
-            vt.csier.table[CSI_CCC][CCC_NOP] = VT_PROC{ p->fork(); };
-            vt.csier.table[CSI_CCC][CCC_IDX] = VT_PROC{ p->fork(q(0)); };
-            vt.csier.table[CSI_CCC][CCC_REF] = VT_PROC{ p->bind(q(0)); };
+
+            #define V [](auto& q, auto& p)
+            //vt.intro[ctrl::nul]             = V{ p->post(utf::frag{ emptyspace, utf::prop{ 0, 1 } }); };
+            vt.intro[ctrl::cr ]              = V{ q.pop_if(ctrl::eol); p->task({ fn::nl,1 }); };
+            vt.intro[ctrl::tab]              = V{ p->task({ fn::tb, q.pop_all(ctrl::tab) }); };
+            vt.intro[ctrl::eol]              = V{ p->task({ fn::nl, q.pop_all(ctrl::eol) }); };
+            vt.csier.table[csi__ed]          = V{ p->task({ fn::ed, q(0) }); }; // CSI Ps J
+            vt.csier.table[csi__el]          = V{ p->task({ fn::el, q(0) }); }; // CSI Ps K
+            vt.csier.table[csi_ccc][ccc_nop] = V{ p->fork(); };
+            vt.csier.table[csi_ccc][ccc_idx] = V{ p->fork(q(0)); };
+            vt.csier.table[csi_ccc][ccc_ref] = V{ p->bind(q(0)); };
+            vt.csier.table_hash[csi_hsh_psh] = V{ p->pushsgr(); }; // CSI # {  Push current SGR attributes and style onto stack.
+            vt.csier.table_hash[csi_hsh_pop] = V{ p->popsgr();  }; // CSI # }  Pop  current SGR attributes and style from stack.
+            #undef V
         }
         page              (view utf8) {          ansi::parse(utf8, this);               }
         auto& operator  = (view utf8) { clear(); ansi::parse(utf8, this); return *this; }
@@ -1548,6 +1577,26 @@ namespace netxs::ui
             parts.emplace(id, *layer);
             return **layer;
         }
+        // page: CSI # {  Push SGR attributes.
+        void pushsgr()
+        {
+            parser::flush();
+            stack.push_back({ parser::style, parser::brush });
+            if (stack.size() == 10) stack.pop_front();
+        }
+        // page: CSI # }  Pop SGR attributes.
+        void popsgr()
+        {
+            parser::flush();
+            if (stack.size())
+            {
+                auto& [s, b] = stack.back();
+                parser::style = s;
+                parser::brush = b;
+                parser::flush_style();
+                stack.pop_back();
+            }
+        }
         // page: Clear the list of paragraphs.
         page& clear(bool preserve_state = faux)
         {
@@ -1595,11 +1644,11 @@ namespace netxs::ui
             stream(publish);
         }
         // page: Split the text run.
-        template<bool FLUSH = true>
+        template<bool Flush = true>
         void fork()
         {
-            if constexpr (FLUSH) parser::flush();
-            layer = batch.insert(std::next(layer), std::make_shared<para>(parser::style));
+            if constexpr (Flush) parser::flush();
+            layer = batch.insert(std::next(layer), ptr::shared<para>(parser::style));
             (**layer).id(++index);
             shrink();
         }
@@ -1641,6 +1690,19 @@ namespace netxs::ui
             auto& item = **layer;
             item.style = parser::style;
         }
+        void post(utf::frag const& cluster)
+        {
+            if (cluster.attr.cdpoint == 0) // Override null character - set a narrow width.
+            {
+                auto c = cluster;
+                c.attr.ucwidth = netxs::unidata::widths::slim;
+                ansi::parser::post(c);
+            }
+            else
+            {
+                ansi::parser::post(cluster);
+            }
+        }
         void data(si32 count, grid const& proto) override
         {
             auto& item = **layer;
@@ -1707,13 +1769,13 @@ namespace netxs::ui
                 data += tag2;
                 data += istr;
             }
-            template<svga VGAMODE = svga::truecolor>
+            template<svga Mode = svga::truecolor>
             auto fgc(rgba const& c)
             {
                 base.inv() ? clr(c, bg_1, bg_2)
                            : clr(c, fg_1, fg_2);
             }
-            template<svga VGAMODE = svga::truecolor>
+            template<svga Mode = svga::truecolor>
             auto bgc(rgba const& c)
             {
                 base.inv() ? clr(c, fg_1, fg_2)
@@ -1869,9 +1931,9 @@ namespace netxs::ui
                     }
                 }
             }
-            template<svga VGAMODE>
+            template<svga Mode>
             auto fgc(rgba const&) { }
-            template<svga VGAMODE>
+            template<svga Mode>
             auto bgc(rgba const&) { }
             auto bld(bool ) { }
             auto itc(bool ) { }
@@ -1944,9 +2006,9 @@ namespace netxs::ui
             {
                 data += utf8;
             }
-            template<svga VGAMODE>
+            template<svga Mode>
             auto fgc(rgba const&) { }
-            template<svga VGAMODE>
+            template<svga Mode>
             auto bgc(rgba const&) { }
             auto bld(bool ) { }
             auto itc(bool ) { }
@@ -1991,30 +2053,26 @@ namespace netxs::ui
 
             // Override vt-functionality.
             using namespace netxs::ansi;
-            vt.intro[ctrl::TAB] = VT_PROC{ p->tabs(q.pop_all(ctrl::TAB)); };
+            #define V [](auto& q, auto& p)
+            vt.intro[ctrl::tab] = V{ p->tabs(q.pop_all(ctrl::tab)); };
+            #undef V
         }
 
         derived_from_page (view utf8) {          ansi::parse(utf8, this);               }
         auto& operator  = (view utf8) { clear(); ansi::parse(utf8, this); return *this; }
         auto& operator += (view utf8) {          ansi::parse(utf8, this); return *this; }
 
-        void tabs(si32) { log("Tabs are not supported"); }
+        void tabs(si32) { if constexpr (debugmode) log(prompt::page, "Tabs not supported"); }
     };
 
-    class tone
+    struct tone
     {
-    public:
-
-        #define PROP_LIST                              \
-        X(fader     , "UI fader duration")             \
-        X(fastfader , "UI fast fader duration")        \
+        #define prop_list                              \
         X(kb_focus  , "Keyboard focus indicator")      \
         X(brighter  , "Highlighter modificator")       \
         X(shadower  , "Darklighter modificator")       \
         X(shadow    , "Light Darklighter modificator") \
-        X(lucidity  , "Global transparency")           \
         X(selector  , "Selection overlay")             \
-        X(bordersz  , "Border size")                   \
         X(highlight , "Hilighted item color")          \
         X(warning   , "Warning color")                 \
         X(danger    , "Danger color")                  \
@@ -2025,13 +2083,13 @@ namespace netxs::ui
         X(menu_black, "Dark menu color")
 
         #define X(a, b) a,
-        enum prop { PROP_LIST count };
+        enum prop { prop_list count };
         #undef X
 
         //#define X(a, b) b,
-        //text description[prop::count] = { PROP_LIST };
+        //text description[prop::count] = { prop_list };
         //#undef X
-        #undef PROP_LIST
+        #undef prop_list
 
         prop active  = prop::brighter;
         prop passive = prop::shadower;
